/**
 * TableRowBuilder
 * 
 * Copyright (c) 2005 Eugene Matyushkin (E-mail: skipy@mail.ru)
 *
 * License agreement:
 *
 * 1. This code is published AS IS. Author is not responsible for any damage that can be
 *    caused by any application that uses this code.
 * 2. Author does not give a garantee, that this code is error free.
 * 3. This code can be used in NON-COMMERCIAL applications AS IS without any special
 *    permission from author.
 * 4. This code can be modified without any special permission from author IF AND ONLY IF
 *    this license agreement will remain unchanged.
 * 5. SPECIAL PERMISSION for this code usage in COMMERCIAL application SHOULD be obtained
 *    from author.
 */
package ru.skipy.ui.components.sft;

/**
 * Table row builder. Creates {@link TableRow} from data array
 *
 * @author Eugene Matyushkin
 * @version 1.0
 */
class TableRowBuilder {

    /**
     * column classes
     */
    private Class[] columnClasses;


    /**
     * Constructs table row builder
     *
     * @param columnClasses column classes
     */
    TableRowBuilder(Class[] columnClasses) {
        this.columnClasses = columnClasses;
    }


    /**
     * Creates {@link TableRow} from data array
     *
     * @param rowData data array
     *
     * @return table row
     *
     * @throws IllegalArgumentException if rowData is <code>null</code>
     * @throws IllegalArgumentException if rowData length not equal to columnClasses length
     * @throws IllegalArgumentException if rowData element's class doesn't math to column class
     */
    TableRow createRow(Object[] rowData) {
        if (rowData == null) {
            throw new IllegalArgumentException("null row data!");
        }
        if (rowData.length != columnClasses.length) {
            throw new IllegalArgumentException("Row data length (" + rowData.length +
                ") doesn't match column number (" + columnClasses.length + ")!");
        }
        for (int i = 0; i < columnClasses.length; i++) {
            if (!columnClasses[i].equals(rowData[i].getClass())) {
                throw new IllegalArgumentException("Row element class (" + rowData[i].getClass().getName() +
                    ") doesn't match column class (" +
                    columnClasses[i].getName() + ") for column " + i + "!");
            }
        }
        return new TableRow(rowData);
    }


    /**
     * Generated id for table row builder
     *
     * @param classes column classes
     *
     * @return id
     */
    static Id getId(Class[] classes) {
        return new Id(classes);
    }


    /**
     * This class represents Id of table row builder. Id is based on column classes -
     * number and order are taken into acount
     */
    static class Id {

        /**
         * column classes
         */
        private Class[] columnClasses;


        /**
         * Constructs id using column classes array
         *
         * @param columnClasses column classes array
         */
        private Id(Class[] columnClasses) {
            this.columnClasses = new Class[columnClasses.length];
            System.arraycopy(columnClasses, 0, this.columnClasses, 0, columnClasses.length);
        }


        /**
         * Returns hashcode for this id
         *
         * @return hashcode
         */
        public int hashCode() {
            int hashCode = 37;
            for (Class clazz : columnClasses) {
                hashCode += clazz.hashCode() * 17;
            }
            return hashCode;
        }


        /**
         * Checks if object passed is equals to this one. Objects are equal if parameter is
         * an Id, column classes lengths are equal and corresponding column classes are equal.
         *
         * @param obj object to be compared with this one
         *
         * @return <code>true</code> if objects are equal; <code>false</code> otherwise.
         */
        public boolean equals(Object obj) {
            if (obj == null) {
                return false;
            }
            if (!(obj instanceof Id)) {
                return false;
            }
            Id id = (Id)obj;
            if (columnClasses.length != id.columnClasses.length) {
                return false;
            }
            for (int i = 0; i < columnClasses.length; i++) {
                if (!columnClasses[i].equals(id.columnClasses[i])) {
                    return false;
                }
            }
            return true;
        }

    }

}
