/**
 * TableRowSorter
 * 
 * Copyright (c) 2005 Eugene Matyushkin (E-mail: skipy@mail.ru)
 *
 * License agreement:
 *
 * 1. This code is published AS IS. Author is not responsible for any damage that can be
 *    caused by any application that uses this code.
 * 2. Author does not give a garantee, that this code is error free.
 * 3. This code can be used in NON-COMMERCIAL applications AS IS without any special
 *    permission from author.
 * 4. This code can be modified without any special permission from author IF AND ONLY IF
 *    this license agreement will remain unchanged.
 * 5. SPECIAL PERMISSION for this code usage in COMMERCIAL application SHOULD be obtained
 *    from author.
 */
package ru.skipy.ui.components.sft;

import java.util.*;

/**
 * TableRowSorter
 *
 * @author Eugene Matyushkin
 */
final class TableRowSorter implements Comparator<TableRow> {

    /**
     * Comparators collection mapped by target object's class
     */
    private Map<Class, Comparator> comparators;

    /**
     * Table model
     */
    private final SnFTableModel model;

    /**
     * Array of comparators for current sort settings
     */
    private Comparator[] currentComparators = new Comparator[0];

    /**
     * Current sort settings
     */
    private ColumnSortPrefs[] sortPrefs = new ColumnSortPrefs[0];


    /**
     * Constructs table rows sorters
     */
    TableRowSorter(SnFTableModel model) {
        if (model == null) {
            throw new IllegalArgumentException("You can't create sorter on null model!");
        }
        this.model = model;
        comparators = new HashMap<Class, Comparator>();
    }


    /**
     * Sets sort preferences
     *
     * @param prefs sort preferences
     *
     * @throws IllegalArgumentException if argument is <code>null</code>
     * @throws IllegalArgumentException if one of columnts is not sortable
     */
    synchronized void setSortPrefs(ColumnSortPrefs[] prefs) {
        if (prefs == null) {
            throw new IllegalArgumentException("null sort prefs!");
        }
        ColumnSortPrefs newSortPrefs[] = new ColumnSortPrefs[prefs.length];
        if (newSortPrefs.length > 0) {
            System.arraycopy(prefs, 0, newSortPrefs, 0, prefs.length);
        }
        Comparator newComparators[] = new Comparator[prefs.length];
        for (int i = 0; i < prefs.length; i++) {
            ColumnSortPrefs pref = prefs[i];
            Class colClass = model.getColumnClass(pref.getColumnIndex());
            if (comparators.containsKey(colClass)) {
                newComparators[i] = comparators.get(colClass);
            } else {
                if (Comparable.class.isAssignableFrom(colClass)) {
                    newComparators[i] = ComparableComparator.INSTANCE;
                } else {
                    throw new IllegalArgumentException("Can't sort by column " + pref.getColumnIndex() + " (" + model.getColumnName(
                        pref.getColumnIndex()) +
                        ") - neither custom comparator is registered nor class " +
                        "is comparable: " + colClass.getName());
                }
            }
        }
        currentComparators = newComparators;
        sortPrefs = newSortPrefs;
    }


    /**
     * Compares two table rows for order.  Returns a negative integer,
     * zero, or a positive integer as the first row is less than, equal
     * to, or greater than the second one.<br>
     *
     * @param tableRow1 the first row to be compared.
     * @param tableRow2 the row object to be compared.
     *
     * @return a negative integer, zero, or a positive integer as the
     *         first argument is less than, equal to, or greater than the
     *         second.
     *
     * @throws ClassCastException if the arguments' types prevent them from
     *                            being compared by this Comparator.
     */
    public synchronized int compare(TableRow tableRow1, TableRow tableRow2) {
        for (int i = 0; i < currentComparators.length; i++) {
            ColumnSortPrefs prefs = sortPrefs[i];
            int result = currentComparators[i].compare(tableRow1.getObjectAt(prefs.getColumnIndex()),
                                                       tableRow2.getObjectAt(prefs.getColumnIndex()));
            if (result != 0) {
                return result * prefs.getSortMode().getSign();
            }
        }
        return 0;
    }


    /**
     * Registers comparator for the class specified.<br>
     * <br>
     * <b>Note: </b> comparator should be parametrized by specified class. I.e., custom
     * comparator for <code>String</code> (which class is <code>String.class</code> or
     * <code>Class&lt;String&gt;</code>) should be declared as <code><b>impements</b>
     * Comparator&lt;String&gt;</code>
     *
     * @param clazz      class for which comparator is registered
     * @param comparator comparator to be registered
     */
    synchronized <T> void registerComparatorForClass(Class<T> clazz, Comparator<T> comparator) {
        comparators.put(clazz, comparator);
    }


    /**
     * Unregisters comparator for the class specified.
     *
     * @param clazz class, which comparator should be unregistered
     */
    synchronized void unregisterComparatorForClass(Class clazz) {
        comparators.remove(clazz);
    }


    /**
     * Checks if comparator for the class specified is registered.
     *
     * @param clazz class, which comparator is looking for
     *
     * @return <code>true</code> if comparator registered;
     *         <code>false</code> otherwise
     */
    synchronized boolean isComparatorRegistered(Class clazz) {
        return comparators.containsKey(clazz);
    }


    /**
     * This class is an implementation of <code>java.util.Comparator</code> for
     * <code>java.lang.Comparable</code> arguments. This implementation was made
     * only for unification of table row comparison mechanism.
     *
     * @author Eugene Matyushkin
     * @version 1.0
     */
    private static final class ComparableComparator implements Comparator<Comparable> {

        /**
         * Static instance of comparator. It should be used instead of creation of
         * new comparator.
         */
        private static final ComparableComparator INSTANCE = new ComparableComparator();


        /**
         * Compares two <code>Comparable</code> arguments. Just calls <code>compareTo</code>
         * on the first argument and passes into the second one.
         *
         * @param comparable1 the first <code>Comparable</code> object to be compared.
         * @param comparable2 the second <code>Comparable</code> object to be compared.
         *
         * @return a negative integer, zero, or a positive integer as the
         *         first argument is less than, equal to, or greater than the
         *         second.
         *
         * @throws ClassCastException if the arguments' types prevent them from
         *                            being compared by this Comparator.
         */
        public int compare(Comparable comparable1, Comparable comparable2) {
            return comparable1.compareTo(comparable2);
        }

    }

}
